@@ -7,16 +7,16 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include <librevenge-stream/librevenge-stream.h>
-#include <cinttypes>
-#include <locale.h>
-#include <math.h>
-#include <set>
-#include <string.h>
-#include <stdlib.h>
-#include "libcdr_utils.h"
-#include "CDRInternalStream.h"
 #include "CMXParser.h"
+
+#include <librevenge-stream/librevenge-stream.h>
+
+#include <math.h>
+#include <stdlib.h>
+#include <utility>
+
+#include "libcdr_utils.h"
+#include "CDRPath.h"
 #include "CDRCollector.h"
 #include "CDRDocumentStructure.h"
 #include "CMXDocumentStructure.h"
@@ -29,6 +29,8 @@
 #define DUMP_IMAGE 0
 #endif
 
+static const int MAX_RECORD_DEPTH = 1 << 10;
+
 namespace
 {
 
@@ -80,7 +82,7 @@ libcdr::CMXParser::~CMXParser()
 
 bool libcdr::CMXParser::parseRecords(librevenge::RVNGInputStream *input, long size, unsigned level)
 {
-  if (!input)
+  if (!input || level > MAX_RECORD_DEPTH)
   {
     return false;
   }
@@ -123,6 +125,8 @@ bool libcdr::CMXParser::parseRecord(librevenge::RVNGInputStream *input, unsigned
 
     if (fourCC == CDR_FOURCC_RIFF || fourCC == CDR_FOURCC_RIFX || fourCC == CDR_FOURCC_LIST)
     {
+      if (length < 4)
+        return false;
 #ifdef DEBUG
       unsigned listType = readU32(input, m_bigEndian);
 #else
@@ -228,7 +232,7 @@ void libcdr::CMXParser::readCMXHeader(librevenge::RVNGInputStream *input)
   for (i = 0; i < 2; i++)
     tmpString.append((char)readU8(input, m_bigEndian));
   CDR_DEBUG_MSG(("CMX Coordinate Size: %s\n", tmpString.cstr()));
-  unsigned short coordSize = (unsigned short)atoi(tmpString.cstr());
+  auto coordSize = (unsigned short)atoi(tmpString.cstr());
   switch (coordSize)
   {
   case 2:
@@ -334,7 +338,7 @@ void libcdr::CMXParser::readDisp(librevenge::RVNGInputStream *input)
 
 const unsigned *libcdr::CMXParser::_getOffsetByType(unsigned short type, const std::map<unsigned short, unsigned> &offsets)
 {
-  std::map<unsigned short, unsigned>::const_iterator iter = offsets.find(type);
+  auto iter = offsets.find(type);
   if (iter != offsets.end())
     return &(iter->second);
   return nullptr;
@@ -345,10 +349,7 @@ void libcdr::CMXParser::readIxmr(librevenge::RVNGInputStream *input)
   unsigned fourCC = readU32(input, m_bigEndian);
   if (CDR_FOURCC_ixmr != fourCC)
     return;
-  unsigned length = readU32(input, m_bigEndian);
-  const unsigned long maxLength = getRemainingLength(input);
-  if (length > maxLength)
-    length = maxLength;
+  readU32(input, m_bigEndian); // Length
 
   readU16(input, m_bigEndian); // Master ID
   readU16(input, m_bigEndian); // Size
@@ -1063,7 +1064,7 @@ bool libcdr::CMXParser::readFill(librevenge::RVNGInputStream *input)
   libcdr::CDRColor color2;
   libcdr::CDRImageFill imageFill;
   libcdr::CDRGradient gradient;
-  unsigned fillId = (unsigned)input->tell();
+  auto fillId = (unsigned)input->tell();
   unsigned fillType = readU16(input, m_bigEndian);
   switch (fillType)
   {
@@ -1374,11 +1375,11 @@ bool libcdr::CMXParser::readFill(librevenge::RVNGInputStream *input)
         {
         case CMX_Tag_RenderAttr_FillSpec_Texture:
         {
-          long subStartOffset = input->tell();
           unsigned char subTagId = 0;
           unsigned short subTagLength = 0;
           do
           {
+            long subStartOffset = input->tell();
             subTagId = readU8(input, m_bigEndian);
             if (subTagId == CMX_Tag_EndTag)
               break;
@@ -1514,7 +1515,7 @@ bool libcdr::CMXParser::readRenderingAttributes(librevenge::RVNGInputStream *inp
   if (bitMask & 0x02) // outline
   {
     CDRLineStyle lineStyle;
-    unsigned lineStyleId = (unsigned)input->tell();
+    auto lineStyleId = (unsigned)input->tell();
     if (m_precision == libcdr::PRECISION_32BIT)
     {
       do
@@ -1653,7 +1654,7 @@ void libcdr::CMXParser::readJumpAbsolute(librevenge::RVNGInputStream *input)
     unsigned short tagLength = 0;
     do
     {
-      long endOffset = input->tell() + tagLength;
+      long offset = input->tell();
       tagId = readU8(input, m_bigEndian);
       if (tagId == CMX_Tag_EndTag)
       {
@@ -1669,7 +1670,7 @@ void libcdr::CMXParser::readJumpAbsolute(librevenge::RVNGInputStream *input)
       default:
         break;
       }
-      input->seek(endOffset, librevenge::RVNG_SEEK_SET);
+      input->seek(offset + tagLength, librevenge::RVNG_SEEK_SET);
     }
     while (tagId != CMX_Tag_EndTag);
   }
@@ -2357,7 +2358,11 @@ libcdr::CDRLineStyle libcdr::CMXParser::getLineStyle(unsigned id)
   if (iterPen != m_parserState.m_pens.end())
   {
     tmpLineStyle.lineWidth = iterPen->second.m_width * (iterPen->second.m_matrix.getScaleX()+iterPen->second.m_matrix.getScaleY())/ 2.0;
-    tmpLineStyle.stretch = iterPen->second.m_aspect * (iterPen->second.m_matrix.getScaleX()/iterPen->second.m_matrix.getScaleY());
+    if (iterPen->second.m_matrix.getScaleY() != 0)
+      tmpLineStyle.stretch = iterPen->second.m_matrix.getScaleX()/iterPen->second.m_matrix.getScaleY();
+    else
+      tmpLineStyle.stretch = 1.0;
+    tmpLineStyle.stretch *= iterPen->second.m_aspect;
     tmpLineStyle.angle = iterPen->second.m_angle;
   }
   std::map<unsigned, std::vector<unsigned> >::const_iterator iterDash = m_parserState.m_dashArrays.find(dashArrayId);
